@@ -223,9 +223,22 @@ class fbrBrowser {
 	{
 		global $gCms;
 		$db = $gCms->GetDb();
+		if (! is_array($params['response_id']))
+			{
+			$params['response_id'] = array($params['response_id']);
+			}
+		
 		$sql = 'DELETE FROM ' . cms_db_prefix().
 				'module_fb_formbrowser where fbr_id=?';
-		$res = $db->Execute($sql, array($params['response_id']));
+		$fb = $this->module_ptr->GetModuleInstance('FormBuilder');
+
+		foreach ($params['response_id'] as $trid)
+			{
+			$res = $db->Execute($sql, array($trid));
+			$parms = array_merge($params,array('response_id'=>$trid));
+			$fb->DeleteFromSearchIndex($parms);
+			}
+
 	}
 
     function LoadResponse($id,&$mod_ptr, &$params, $which_list='full_fields', $adminside=false)
@@ -236,6 +249,7 @@ class fbrBrowser {
 		$flds = $this->FieldOrderList($which_list);
 
 		$fbf = $fb->GetFormBrowserField($this->FormId);
+
 		if ($fbf != false)
 			{
 			// if we're binding to FEU, get the FEU ID, see if there's a response for
@@ -292,7 +306,7 @@ class fbrBrowser {
 		$mod_ptr->smarty->assign('title_approval_date',$mod_ptr->Lang('title_approval_date'));
 		$mod_ptr->smarty->assign('title_user_approved',$mod_ptr->Lang('title_user_approved'));
 		$mod_ptr->smarty->assign('resp',$response);
-		$mod_ptr->smarty->assign('count',count($response->names)+1);
+		$mod_ptr->smarty->assign('count',count($response->names));
 		$mod_ptr->smarty->assign('browser_title',$this->GetName());
 		$mod_ptr->smarty->assign('fbrp_message','');
 		foreach ($response->names as $id=>$name)
@@ -305,6 +319,7 @@ class fbrBrowser {
  			$mod_ptr->smarty->assign($name,$obj);
 			$mod_ptr->smarty->assign('fld_'.$id,$obj);
 			}
+		return $response;
     }   
 
   function MakeVar($string)
@@ -345,13 +360,25 @@ class fbrBrowser {
 	{
 		global $gCms;
 		$count = 0;
+		
+		if ($which_list == 'admin_list_fields' && $this->GetAttr('browser_type') == 'advanced')
+			{
+			$flds = $this->FieldOrderList('admin_full_fields');
+			}
+		else if ($which_list == 'list_fields' && $this->GetAttr('browser_type') == 'advanced')
+			{
+			$flds = $this->FieldOrderList('full_fields');
+			}
+		else
+			{
+			$flds = $this->FieldOrderList($which_list);
+			}
 
-		$flds = $this->FieldOrderList($which_list);
 		$fb = $mod_ptr->GetModuleInstance('FormBuilder');
 		$reqAdminApproval = $this->GetAttr('require_admin_approval','0')=='0'?false:true;
 		$reqUserApproval = $this->GetAttr('require_user_approval','0')=='0'?false:true;
 		$sortField = isset($params['fbrp_sort_field'])?$params['fbrp_sort_field']:'';
-		if (empty($params['fbr_sort_dir']))
+		if (empty($params['fbrp_sort_dir']))
 			{
 			$sortDir = 'd';
 			$newSortDir = 'a';
@@ -377,7 +404,6 @@ class fbrBrowser {
 			list($count,$names,$vals) = $fb->GetSortedResponses($this->FormId,(($this->Page - 1) * $perPage),
 				$perPage, false, $reqUserApproval,$flds,$mod_ptr->GetPreference('date_format','d F y'),$params);
 				
-				
 			// create sortable names
 			foreach($names as $i=>$nval)
 				{
@@ -441,7 +467,8 @@ class fbrBrowser {
 						$gCms->variables['admintheme']->DisplayImage('icons/system/view.gif',$mod_ptr->Lang('view'),'','','systemicon'),
 						array('response_id'=>$vals[$i]->id,'form_id'=>$this->FormId,'browser_id'=>$this->Id,'fbrp_page'=>$this->Page,
 							'fbrp_sort_field'=>$sortField,'fbrp_sort_dir'=>$sortDir));
-
+					$vals[$i]->delbox = $mod_ptr->CreateInputCheckbox($id,
+							'response_id[]',$vals[$i]->id);
 				}
 			$mod_ptr->smarty->assign('addlink',$mod_ptr->CreateLink($id,
 						'admin_edit_resp', '',
@@ -458,18 +485,9 @@ class fbrBrowser {
 			{
 			$perPage = $this->GetAttr('rows_per_page','10');
 
-//			if (isset($params['fbrp_sort_field']))
-//				{
-				list($count,$names,$vals) = $fb->GetSortedResponses($this->FormId,(($this->Page-1) * $perPage),
+			list($count,$names,$vals) = $fb->GetSortedResponses($this->FormId,(($this->Page-1) * $perPage),
 					$perPage, $reqAdminApproval, $reqUserApproval,$flds,$mod_ptr->GetPreference('date_format','d F y'), $params);
-//				}
-//			else
-//				{
-//				list($count,$names,$vals) = $fb->GetResponses($this->FormId,(($this->Page-1) * $perPage),
-// Start ALBY
-// $perPage, $reqAdminApproval, $reqUserApproval,$flds,$mod_ptr->GetPreference('date_format','d F y'), $params);
-// End ALBY
-//				}
+
 			// create sortable names
 			foreach($names as $i=>$nval)
 				{
@@ -629,6 +647,15 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 			{
 			$mod_ptr->smarty->assign('hasnav',0);
 			}
+			
+		if ($this->GetAttr('udt_name','') != '')
+			{
+			// call UDT
+			$parms = array('vals'=>&$vals,'names'=>&$names, 'sortingnames'=>&$sortingnames, 'side'=>($adminside?'admin':'user'));
+		    $usertagops = $gCms->GetUserTagOperations();
+		    $res = $usertagops->CallUserTag( $this->GetAttr('udt_name'), $parms);
+			}
+
 		$mod_ptr->smarty->assign('adminapproval',$reqAdminApproval?1:0);
 		$mod_ptr->smarty->assign('userapproval',$reqUserApproval?1:0);
 		$mod_ptr->smarty->assign('list',$vals);
@@ -689,6 +716,21 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 		return $outstr;
 	}
 
+	function BrowserShowListXLSFile(&$mod_ptr,&$params)
+	{
+		global $gCms;
+		$config = $gCms->GetConfig();
+		$count = 0;
+
+		$fb = $mod_ptr->GetModuleInstance('FormBuilder');
+		$flds = array();
+		$parms = array();			
+		$strip = ($mod_ptr->GetPreference('strip_on_export','0') == '1');
+		$filespec = $config['uploads_path'].DIRECTORY_SEPARATOR.$params['filespec'];
+		return $fb->WriteSortedResponsesToFile($this->FormId, $filespec, $strip, $mod_ptr->GetPreference('date_format','d F y'), $parms);
+	}
+
+
     function LoadBrowser($loadDeep=false)
     {
     	$tmp = array();
@@ -883,22 +925,23 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 		return true;
     }
 
-	function AddEditBrowser($id, $returnid, $message='')
+	function AddEditBrowser($id, $returnid, $tab, $message='')
 	{
 	$mod = $this->module_ptr;
+	global $gCms;
 		$mod->smarty->assign('fbrp_message',$message);
 		$mod->smarty->assign('formstart',
 			$mod->CreateFormStart($id, 'admin_store_browser', $returnid));
 		$mod->smarty->assign('browser_id',
 			$mod->CreateInputHidden($id, 'browser_id', $this->Id));
 		$mod->smarty->assign('tab_start',$mod->StartTabHeaders().
-			$mod->SetTabHeader('maintab',$mod->Lang('tab_main')).
-			$mod->SetTabHeader('useroptiontab',$mod->Lang('tab_user_options')).
-			$mod->SetTabHeader('adminoptiontab',$mod->Lang('tab_admin_options')).			
-			$mod->SetTabHeader('ulisttab',$mod->Lang('tab_list_user_template')).			
-			$mod->SetTabHeader('ufulltab',$mod->Lang('tab_full_user_template')).			
-			$mod->SetTabHeader('alisttab',$mod->Lang('tab_list_admin_template')).			
-			$mod->SetTabHeader('afulltab',$mod->Lang('tab_full_admin_template')).			
+			$mod->SetTabHeader('maintab',$mod->Lang('tab_main'),('maintab' == $tab)?true:false).
+			$mod->SetTabHeader('useroptiontab',$mod->Lang('tab_user_options'),('useroptiontab' == $tab)?true:false).
+			$mod->SetTabHeader('adminoptiontab',$mod->Lang('tab_admin_options'),('adminoptiontab' == $tab)?true:false).			
+			$mod->SetTabHeader('ulisttab',$mod->Lang('title_browser_user_list_template'),('ulisttab' == $tab)?true:false).			
+			$mod->SetTabHeader('ufulltab',$mod->Lang('title_browser_user_full_template'),('ufulltab' == $tab)?true:false).			
+			$mod->SetTabHeader('alisttab',$mod->Lang('title_browser_admin_list_template'),('alisttab' == $tab)?true:false).			
+			$mod->SetTabHeader('afulltab',$mod->Lang('title_browser_admin_full_template'),('afulltab' == $tab)?true:false).			
 			$mod->EndTabHeaders() . $mod->StartTabContent());
 		$mod->smarty->assign('tabs_end',$mod->EndTabContent());
 		$mod->smarty->assign('maintab_start',$mod->StartTab("maintab"));
@@ -910,7 +953,45 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 		$mod->smarty->assign('afulltab_start',$mod->StartTab("afulltab"));
 		$mod->smarty->assign('tab_end',$mod->EndTab());
 		$mod->smarty->assign('form_end',$mod->CreateFormEnd());
+		
+		// Stikki's version of this code :)
+		$mod->smarty->assign('title_load_template',$mod->Lang('title_load_template'));
+		$modLink = $mod->CreateLink($id, 'admin_get_template', $returnid, '', array(), '', true);
+		$mod->smarty->assign('security_key',CMS_SECURE_PARAM_NAME.'='.$_SESSION[CMS_USER_KEY]);
+
+		$templateList = array(''=>'',$mod->Lang('default_admin_list_template')=>'admin_browse_list.tpl',
+			$mod->Lang('default_admin_full_template')=>'admin_browse_resp.tpl',
+			$mod->Lang('default_user_list_template')=>'user_browse_list.tpl',
+			$mod->Lang('default_user_full_template')=>'user_browse_resp.tpl');
+
+		$allBrowsers = $mod->GetBrowsers();
+		foreach ($allBrowsers as $thisBrowser)
+			{
+			if ($thisBrowser['browser_id'] != $this->Id)
+				{
+				$templateList[$mod->Lang('title_browser_admin_list_template').' '.$mod->Lang('browser_template_name',array($thisBrowser['name']))] =
+					$thisBrowser['browser_id'].'.admin_list';
+				$templateList[$mod->Lang('title_browser_admin_full_template').' '.$mod->Lang('browser_template_name',array($thisBrowser['name']))] =
+					$thisBrowser['browser_id'].'.admin_full';
+				$templateList[$mod->Lang('title_browser_user_list_template').' '.$mod->Lang('browser_template_name',array($thisBrowser['name']))] =
+					$thisBrowser['browser_id'].'.user_list';
+				$templateList[$mod->Lang('title_browser_user_full_template').' '.$mod->Lang('browser_template_name',array($thisBrowser['name']))] =
+					$thisBrowser['browser_id'].'.user_full';
+				}
+			}
+			
+		$mod->smarty->assign('input_load_ul_template',$mod->CreateInputDropdown($id,
+			'fbrp_fbr_ul_template_load', $templateList, -1, '', 'id="fbr_ul_template_load" onchange="jQuery(this).fbr_get_template(\''.$mod->Lang('template_are_you_sure').'\',\''.$modLink.'\',\'#fbr_user_list_template\');"'));
+		$mod->smarty->assign('input_load_uf_template',$mod->CreateInputDropdown($id,
+			'fbrp_fbr_uf_template_load', $templateList, -1, '', 'id="fbr_uf_template_load" onchange="jQuery(this).fbr_get_template(\''.$mod->Lang('template_are_you_sure').'\',\''.$modLink.'\',\'#fbr_user_full_template\');"'));
+		$mod->smarty->assign('input_load_al_template',$mod->CreateInputDropdown($id,
+			'fbrp_fbr_al_template_load', $templateList, -1, '', 'id="fbr_al_template_load" onchange="jQuery(this).fbr_get_template(\''.$mod->Lang('template_are_you_sure').'\',\''.$modLink.'\',\'#fbr_admin_list_template\');"'));
+		$mod->smarty->assign('input_load_af_template',$mod->CreateInputDropdown($id,
+			'fbrp_fbr_af_template_load', $templateList, -1, '', 'id="fbr_af_template_load" onchange="jQuery(this).fbr_get_template(\''.$mod->Lang('template_are_you_sure').'\',\''.$modLink.'\',\'#fbr_admin_full_template\');"'));
+
+			
 		$mod->smarty->assign('title_browser_name',$mod->Lang('title_browser_name'));
+		$mod->smarty->assign('title_browser_search_field',$mod->Lang('title_browser_search_field'));
 		$mod->smarty->assign('input_browser_name',
 			$mod->CreateInputText($id, 'fbrp_browser_name',
 			$this->Name, 50));
@@ -943,9 +1024,9 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 
 		if($this->Id > 0)
     	{
-    		$mod->smarty->assign('submit_button',
+    		$mod->smarty->assign('submit_button',$mod->CreateInputHidden($id, 'active_tab', '', 'id="fbr_atab"').
     			$mod->CreateInputSubmit($id, 'fbrp_submit',
-    			$mod->Lang('save_and_continue')));
+    			$mod->Lang('save_and_continue'),'onclick="jQuery(this).fbr_set_tab()"'));
     		$mod->smarty->assign('hidden',
     			$mod->CreateInputHidden($id, 'fbrp_browser_op',$mod->Lang('updated')));
 			$mod->smarty->assign('adding',0);
@@ -956,19 +1037,29 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 			$mod->smarty->assign('input_form_id',$form->GetName());
 			$mod->smarty->assign('input_browser_type',
             ($this->GetAttr('browser_type','simple')=='simple'?$mod->Lang('simple_browser'):$mod->Lang('advanced_browser')));
-            
+
+		 $field_names = array();
+		 $fields = $form->GetFields();
+		 for ($i=0;$i<count($fields);$i++)
+				{
+				if ($fields[$i]->DisplayInSubmission())
+					{
+					$field_names[$fields[$i]->GetName()]=$fields[$i]->GetName();
+					}
+				}
+					
          if ($this->GetAttr('browser_type','simple')!='simple' &&
             $this->GetAttr('fs','') == '')
             {
             $mod->smarty->assign('savemsg',
                $mod->CreateInputHidden($id, 'fbrp_browsera_fs','1'));
             $mod->smarty->assign('fbrp_message',$mod->Lang('must_save'));
-            }
+            
+			}
          else
             {
             $mod->smarty->assign('savemsg','');
             }
-			$fields = $form->GetFields();
 			$field_count = 0;
 			for ($i=0;$i<count($fields);$i++)
 				{
@@ -992,17 +1083,9 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 					$onerow->name = $fields[$i]->GetName();
 					$onerow->id = $fields[$i]->GetId();
             $ulf = (isset($this->listFields[$fields[$i]->GetId()])?$this->listFields[$fields[$i]->GetId()]:'-1');
-				if (empty($ulf) || ($ulf == -1 && $incr<3))
-               {
-               $ulf = $incr;
-               }
-            $alf = (isset($this->adminListFields[$fields[$i]->GetId()])?$this->adminListFields[$fields[$i]->GetId()]:'-1');
-				if (empty($alf) || ($alf == -1 && $incr<3))
-               {
-               $alf = $incr;
-               }
-            $uff = (isset($this->fullFields[$fields[$i]->GetId()])?$this->fullFields[$fields[$i]->GetId()]:$incr);
-            $aff = (isset($this->adminFullFields[$fields[$i]->GetId()])?$this->adminFullFields[$fields[$i]->GetId()]:$incr);
+           $alf = (isset($this->adminListFields[$fields[$i]->GetId()])?$this->adminListFields[$fields[$i]->GetId()]:'-1');
+            $uff = (isset($this->fullFields[$fields[$i]->GetId()])?$this->fullFields[$fields[$i]->GetId()]:'-1');
+            $aff = (isset($this->adminFullFields[$fields[$i]->GetId()])?$this->adminFullFields[$fields[$i]->GetId()]:'-1');
 
          if ($this->GetAttr('browser_type','simple')=='simple')
             {
@@ -1010,7 +1093,7 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 				$onerow->admin_list_order = $mod->CreateInputDropdown($id, 'fbrp_admin_list_field_'.$fields[$i]->GetId(),$order, -1, $alf);
 				$onerow->full_order = $mod->CreateInputDropdown($id, 'fbrp_full_field_'.$fields[$i]->GetId(),$order, -1, $uff);
 				$onerow->admin_full_order = $mod->CreateInputDropdown($id, 'fbrp_admin_full_field_'.$fields[$i]->GetId(),$order, -1, $aff);
-				}
+			}
          else
             {
             // annoying bug requires explicit string cast for zeros.
@@ -1034,9 +1117,19 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 			$mod->smarty->assign('title_admin_full_order', $mod->Lang('title_admin_full_order'));
 
 			}
+			$mod->smarty->assign('toggle_user_list','fbrp_list_field_');
+			$mod->smarty->assign('toggle_user_full','fbrp_full_field_');
+			$mod->smarty->assign('toggle_admin_list','fbrp_admin_list_field_');
+			$mod->smarty->assign('toggle_admin_full','fbrp_admin_full_field_');
+			$mod->smarty->assign('toggle_column',$mod->Lang('select_deselect_all'));
+			$mod->smarty->assign('id',$id);
+			
 			$mod->smarty->assign('field_count',$field_count);
 			$mod->smarty->assign('title_field_count', $mod->Lang('title_field_count',$field_count));			
 
+			$mod->smarty->assign('input_browser_search_field',$mod->CreateInputDropdown($id,'fbrp_browsera_search_field',$field_names,-1,$this->GetAttr('search_field','')));
+
+
 			if ($this->GetAttr('browser_type','simple')=='simple')
             {
             $mod->smarty->assign('mode_simple','1');
@@ -1069,12 +1162,26 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
             array($mod->Lang('simple_browser')=>'simple',$mod->Lang('advanced_browser')=>'advanced')).
             '<br />'.$mod->Lang('type_help'));
 		}
+		
+	    $usertagops = $gCms->GetUserTagOperations();
+	    $usertags = $usertagops->ListUserTags();
+	    $usertaglist = array($mod->Lang('none')=>'');
+	    foreach( $usertags as $key => $value )
+			{
+			$usertaglist[$value] = $key;
+			}
+		$mod->smarty->assign('input_udt_name',
+			    $mod->CreateInputDropdown($id,'fbrp_browsera_udt_name',$usertaglist,-1,$this->GetAttr('udt_name','')));
+		$mod->smarty->assign('title_udt_name',$mod->Lang('title_udt_name'));
+	    
+		
 		$rowsOptions = array('1'=>'1','2'=>'2','3'=>'3','4'=>'4','5'=>'5',
 			'6'=>'6','7'=>'7','8'=>'8','9'=>'9','10'=>'10',
 			'11'=>'11','12'=>'12','13'=>'13','14'=>'14','15'=>'15',
 			'16'=>'16','17'=>'17','18'=>'18','19'=>'19','20'=>'20',
 			'25'=>'25','30'=>'30','35'=>'35','40'=>'40','45'=>'45',
-			'50'=>'50','60'=>'60','70'=>'70','80'=>'80','90'=>'90','100'=>'100');
+			'50'=>'50','60'=>'60','70'=>'70','80'=>'80','90'=>'90','100'=>'100',
+			'500'=>'500','1000'=>'1000','5000'=>'5000');
 		$mod->smarty->assign('title_rows_per_page',$mod->Lang('title_rows_per_page'));
 		$mod->smarty->assign('input_rows_per_page',$mod->CreateInputDropdown($id,'fbrp_browsera_rows_per_page',$rowsOptions,-1,$this->GetAttr('rows_per_page','10')));
 		$mod->smarty->assign('title_admin_rows_per_page',$mod->Lang('title_admin_rows_per_page'));
@@ -1096,20 +1203,23 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
 
 		$mod->smarty->assign('input_browser_user_list_template',
 			$mod->CreateTextArea(false, $id,
-				$this->GetAttr('user_list_template',$this->DefaultUserListTemplate()), 'fbrp_browsera_user_list_template'));
+				$this->GetAttr('user_list_template',$this->DefaultUserListTemplate()), 'fbrp_browsera_user_list_template','','fbr_user_list_template',
+				'', '', '80', '15','','html'));
 		$mod->smarty->assign('input_browser_user_full_template',
 			$mod->CreateTextArea(false, $id,
-				$this->GetAttr('user_full_template',$this->DefaultUserFullTemplate()), 'fbrp_browsera_user_full_template'));
+				$this->GetAttr('user_full_template',$this->DefaultUserFullTemplate()), 'fbrp_browsera_user_full_template','','fbr_user_full_template',
+				'', '', '80', '15','','html'));
 		$mod->smarty->assign('input_browser_admin_list_template',
 			$mod->CreateTextArea(false, $id,
-				$this->GetAttr('admin_list_template',$this->DefaultAdminListTemplate()), 'fbrp_browsera_admin_list_template'));
+				$this->GetAttr('admin_list_template',$this->DefaultAdminListTemplate()), 'fbrp_browsera_admin_list_template','','fbr_admin_list_template',
+				'', '', '80', '15','','html'));
 		$mod->smarty->assign('input_browser_admin_full_template',
 			$mod->CreateTextArea(false, $id,
-				$this->GetAttr('admin_full_template',$this->DefaultAdminFullTemplate()), 'fbrp_browsera_admin_full_template'));
+				$this->GetAttr('admin_full_template',$this->DefaultAdminFullTemplate()), 'fbrp_browsera_admin_full_template','','fbr_admin_full_template',
+				'', '', '80', '15','','html'));
         return $mod->ProcessTemplate('AddEditBrowser.tpl');
 	}
 
-
     function MakeAlias($string, $isForm=false)
     {
     	$string = trim(htmlspecialchars($string));
@@ -1127,12 +1237,12 @@ array_merge($_searchfield, array('fbrp_page'=>($this->Page + 1),'form_id'=>$this
    
 	function DefaultUserListTemplate()
 	{
-		return file_get_contents(dirname(__FILE__).'/../templates/BrowserListDefault.tpl');
+		return file_get_contents(dirname(__FILE__).'/../templates/user_browse_list.tpl');
 	}
 
 	function DefaultUserFullTemplate()
 	{
-		return file_get_contents(dirname(__FILE__).'/../templates/BrowserFullDefault.tpl');
+		return file_get_contents(dirname(__FILE__).'/../templates/user_browse_resp.tpl');
 	}
 
 	function DefaultAdminListTemplate()
