@@ -42,6 +42,8 @@ if (! $this->CheckAccess()) exit;
 		$this->CreateEvent( 'OnFormBrowserRecordDelete' );
 		$this->CreateEvent( 'OnFormBrowserRecordAdd' );
 		$this->CreateEvent( 'OnFormBrowserRecordView' );
+		$this->CreateEvent( 'OnFormBrowserRecordEditPostSave' );
+		$this->CreateEvent( 'OnFormBrowserRecordAddPostSave' );
 
 
 
