@@ -11,6 +11,12 @@ if (! $this->CheckAccess()) exit;
 		$current_version = $oldversion;
 		switch($current_version)
 		{
+			case '0.1':
+			case '0.2':
+			case '0.3':
+				$this->CreateEvent( 'OnFormBrowserRecordEditPostSave' );
+				$this->CreateEvent( 'OnFormBrowserRecordAddPostSave' );
+				
 		}
 		
 		// put mention into the admin log
